#!/usr/bin/env rexx
/*
	This project is based on example 99 of the bsf4oorexx examples, orginally built by Rony G. Flatscher
*/
parse version "_" v "."
parse source  . . pgm
if v<5 then
do
   parse version v
   .error~say(pp(pgm)" needs ooRexx 5.0 or higher, found:" pp(v))
   .bsf.dialog~messageBox(pp(pgm)" needs ooRexx 5.0 or higher," "0a"x || "found:" pp(v), -
                          "Wrong version of ooRexx", "error")
   exit -1
end
call directory filespec('L', pgm)


.environment~my.app=.directory~new
.my.app~bDebug=.false               

call bsf.import "javafx.fxml.FXMLLoader",                      "fx.FXMLLoader"
call bsf.import "javafx.scene.Scene",                          "fx.Scene"
call bsf.import "javafx.beans.property.SimpleStringProperty",  "fx.SimpleStringProperty"
call bsf.import "javafx.beans.property.SimpleIntegerProperty", "fx.SimpleIntegerProperty"

call bsf.import "javafx.collections.FXCollections",            "fx.FXCollections"
call bsf.import "javafx.stage.Modality",                       "fx.Modality"

rexxApp=.RxMainApplication~new

   rexxApp~personFilePath=strip(jsonFile)
.my.app~mainApp=rexxApp

jRexxApp=BsfCreateRexxProxy(rexxApp, ,"javafx.application.Application")

signal on syntax
jRexxApp~launch(jRexxApp~getClass, .nil)
call sysSleep 0.01
exit

syntax:
   co=condition("object")
   say ppJavaExceptionChain(co, .true)
   say " done. "~center(100, "-")
   exit -1

::requires "BSF.CLS"

::class RxMainApplication

::attribute personData

::attribute primaryStage

::attribute personFilePath


::method init
  expose personData prefs personFilePath

  personData=.fx.FXCollections~observableArrayList

        url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	rs=statement~executeQuery("SELECT * FROM members")
	do while rs~next 
		personData~add(.person~new(rs~getString("id"), rs~getString("name")))
 	end
	connection~close

::method start 
  expose locale primaryStage personFilePath
  use arg primaryStage

  bAlertsAvailable=(.java.lang.system~getProperty("javafx.runtime.version")>8)
  .my.app~bDialogsAlertsAvailable=bAlertsAvailable
  if bAlertsAvailable then
  do
      call bsf.import "javafx.scene.control.Alert",                  "fx.Alert"
      call bsf.import "javafx.scene.control.Alert$AlertType",        "fx.Alert.Type"
  end

	primaryStage~setTitle("Jadmin | Member Administration Tool")

  rootLayoutUrl=.bsf~new("java.net.URL", "file:RootLayout.fxml")
  rootLayout   =.fx.FXMLLoader~load(rootLayoutUrl)
  .rootLayoutController~new(self)

  scene=.fx.scene~new(rootLayout)
  primaryStage~setScene(scene)

  overviewUrl=.bsf~new("java.net.URL", "file:PersonOverview.fxml")
  overviewPage=.fx.FXMLLoader~load(overviewUrl) 
  .my.app~overviewPage=overviewPage
  rootLayout~setCenter(overviewPage)
  .my.app~personOverviewController=.PersonOverviewController~new

  primaryStage~show


::method showPersonEditDialog
  expose pedDialog pedController primaryStage
  use arg person

  if \var("pedDIALOG") then 
  do
     personEditDialogUrl=.bsf~new("java.net.URL", "file:PersonEditDialog.fxml")
     page  = .fx.FXMLLoader~load(personEditDialogUrl)
     scene = .fx.Scene~new(page)

     -- AnchorPane page = (AnchorPane) loader.load();
     dialogStage = .bsf~new("javafx.stage.Stage")
     dialogStage~setTitle("Jadmin | Edit Member")
     dialogStage~initModality(.fx.Modality~WINDOW_MODAL)

     dialogStage~initOwner(primaryStage)
     dialogStage~setScene(scene)

     pedController = .PersonEditDialogController~new
     pedController~dialogStage=dialogStage
  end

  pedController~setPerson(person)
  pedController~okClicked=.false


  dialogStage~showAndWait
  return pedController~OkClicked

::class PersonOverviewController
::attribute personTable
::attribute idColumn
::attribute nameColumn

::method init
  expose personTable idColumn nameColumn idLabel nameLabel -
                     teamLabel positionLabel birthdayLabel  -
                     btnNew btnEdit btnDelete

  poDir=.my.app~personOverview.fxml
  personTable    =poDir~personTable
  idColumn=poDir~idColumn
  nameColumn =poDir~nameColumn

  idLabel =poDir~idLabel
  nameLabel  =poDir~nameLabel
  teamLabel=poDir~teamLabel
  positionLabel      =poDir~positionLabel
  birthdayLabel  =poDir~birthdayLabel

  btnNew         =poDir~btnNew
  btnEdit        =poDir~btnEdit
  btnDelete      =poDir~btnDelete

  rp=BSFCreateRexxProxy(self, ,"javafx.beans.value.ChangeListener", "javafx.event.EventHandler")

  personTable~setOnMouseClicked(rp)

  idColumn~setCellValueFactory(BsfCreateRexxProxy(.PropertyValueFactory~new("idProperty"), ,"javafx.util.Callback"))
  nameColumn ~setCellValueFactory(BsfCreateRexxProxy(.PropertyValueFactory~new("nameProperty"),  ,"javafx.util.Callback"))


  self~showPersonDetails(.nil) 
  personTable~setColumnResizePolicy(personTable~CONSTRAINED_RESIZE_POLICY)

  personTable~getSelectionModel ~selectedItemProperty ~addListener(rp)

  btnNew   ~setOnAction(rp)
  btnEdit  ~setOnAction(rp)
  btnDelete~setOnAction(rp)

  self~setItems


::method setItems
  expose personTable
  personTable~setItems(.my.app~mainApp~personData)

::method showPersonDetails
  expose idLabel nameLabel teamLabel positionLabel birthdayLabel exNa
  use arg p
  if p=.nil then
  do
     idLabel  ~setText("")
     nameLabel   ~setText("")
     teamLabel ~setText("")
     positionLabel       ~setText("")
     birthdayLabel   ~setText("")
  end
  else 
  do
     idLabel  ~setText(p~id)
     nameLabel   ~setText(p~name)
     exNa = p~id
	url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	rs=statement~executeQuery("SELECT * FROM members WHERE id = '"exNa"'")
	
     teamLabel ~setText(rs~getString("team"))
     positionLabel       ~setText(rs~getString("position"))
     birthdayLabel   ~setText(rs~getString("birthday"))
connection~close
  end


::method handleDeletePerson
  expose personTable exNa
  selectedIndex=personTable~getSelectionModel~getSelectedIndex

  if selectedIndex >= 0 then
  do
	url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	rs=statement~executeUpdate("DELETE FROM members WHERE id = '"exNa"'")
	connection~close
     p=personTable~getItems~bsf.invokeStrict("remove", "int", selectedIndex)
  end
  else
  do
      if .my.app~bDialogsAlertsAvailable then
      do
         alert=.fx.alert~new(.fx.Alert.Type~WARNING) 

         alert~setTitle("Jadmin | Please select a member")
         alert~setHeaderText(.nil)
         alert~setContentText("Please select a member from the table.")
         alert~showAndWait
      end
      else  -- use BSF.CLS' .bsf.dialog utility class
      do
        .bsf.dialog~messageBox("Please select a member in the table.", "Jadmin | Please select a member", "warning")
      end
  end


::method handleNewPerson
  tempPerson=.Person~new

  okClicked = .my.app~mainApp~showPersonEditDialog(tempPerson)
  if okClicked then
     .my.app~mainApp~personData~add(tempPerson)
	url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	maxId=statement~executeQuery("SELECT * FROM members ORDER BY id DESC LIMIT 1")
	if maxId~next=0 then vid = 0 
	else vid=maxId~getInt("id")
	statement~executeUpdate("INSERT INTO members VALUES('"vid+1"', '"tempPerson~name"', '"tempPerson~birthday"', '"tempPerson~team"', '"tempPerson~position"')")
	tempPerson~id=vid+1
	connection~close

::method handleEditPerson
  expose personTable

  selectedPerson = personTable~getSelectionModel~getSelectedItem
  if selectedPerson <> .nil then
  do
      person=BSFRexxProxy(selectedPerson)
	okClicked = .my.app~mainApp~showPersonEditDialog(person)
      if okClicked=.true then
          self~showPersonDetails(person)
  end
  else
  do
      if .my.app~bDialogsAlertsAvailable then
      do
         alert=.fx.alert~new(.fx.Alert.Type~WARNING)

       alert~setTitle("Jadmin | Please select a member")
         alert~setHeaderText(.nil)
         alert~setContentText("Please select a member in the table.")
         alert~showAndWait
      end
      else  -- use BSF.CLS' .bsf.dialog utility class
      do
        .bsf.dialog~messageBox("Please select a member in the table.", "Jadmin | Please select a member", "warning")
      end
  end

::method changed unguarded   
  use arg observable, oldValue, newValue
  self~showPersonDetails(newValue)

::method handle 
  expose btnNew btnEdit btnDelete personTable
  use arg event

  tgtObjectName=event~getTarget~objectname
  select
     when tgtObjectName=btnNew~objectName    then self~handleNewPerson
     when tgtObjectName=btnEdit~objectName   then self~handleEditPerson
     when tgtObjectName=btnDelete~objectName then self~handleDeletePerson
     otherwise
     do
        if event~getSource~objectName=personTable~objectName then
        do
            if event~getClickCount>1 then 
               self~handleEditPerson
        end
        else 
        do
           .error~say(self"::handle, UNKNOWN event:" pp(event~toString))
        end
     end
  end


::class PropertyValueFactory
::method init
  expose  propName   --handler -- name of property getter method
  use strict arg propName -- , handler

::method call
  expose propName   -- handler
  use arg o        
  return BsfRexxProxy(o~getValue)~send(propName)


::class PersonEditDialogController
::attribute okClicked
::attribute dialogStage
::attribute person        

::method init
  expose nameField teamField positionField birthdayField -
         btnOK btnCancel -
         okClicked

  okClicked=.false

  
  pedDir=.my.app~personEditDialog.fxml
  nameField  =pedDir~nameField
  streetField    =pedDir~streetField
  teamField=pedDir~teamField
  positionField      =pedDir~positionField
  birthdayField  =pedDir~birthdayField

  btnOK          =pedDir~btnOK
  btnCancel      =pedDir~btnCancel

   
  rp=BSFCreateRexxProxy(self, ,"javafx.event.EventHandler")
  
  btnOK    ~setOnAction(rp)
  btnCancel~setOnAction(rp)


::method handle   
  expose btnOK btnCancel
  use arg event

  tgtObjectName=event~getTarget~objectname
  select
     when tgtObjectName=btnOK~objectName     then self~handleOK
     when tgtObjectName=btnCancel~objectName then self~handleCancel
     otherwise .error~say("UNKNOWN target:" pp(event~getTarget) pp(event~getTarget~tostring))
  end


::method setPerson
  expose idField nameField streetField teamField positionField birthdayField -
         person
  use arg person 

	exId = person~id
	url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	rs=statement~executeQuery("SELECT * FROM members where id = '"exId"'")
if rs~next then do
  nameField  ~setText(rs~getString("name"))
  teamField~setText(rs~getString("team"))
  positionField      ~setText(rs~getString("position"))
  birthdayField  ~setText(rs~getString("birthday"))
  birthdayField  ~setPromptText("dd-mm-yyyy")
end 
else do
  nameField  ~setText(person~name)
  teamField~setText(person~team)
  positionField      ~setText(person~position)
  birthdayField  ~setText(person~birthday)
  birthdayField  ~setPromptText("dd-mm-yyyy")
end
connection~close

::method handleCancel
  expose dialogStage

  dialogStage~hide


::method handleOK
  expose nameField streetField teamField positionField birthdayField -
         person okClicked dialogStage

  if (self~isInputValid) then
  do
      person~name   = nameField  ~getText
      person~team = teamField~getText
      person~position       = positionField      ~getText
      person~birthday   = birthdayField  ~getText
      okClicked = .true
      exId = person~id
	url="jdbc:sqlite:jadmin.db"
	connection=bsf.import("java.sql.DriverManager")~getConnection(url)
	statement =connection~createStatement
	statement~setQueryTimeout(30)
	rs=statement~executeUpdate("UPDATE members SET name = '"person~name"', birthday = '"person~birthday"', team = '"person~team"', position = '"person~position"' where id = '"exId"'")
	connection~close
      dialogStage~hide
  end

::method isInputValid
  expose idField nameField streetField teamField positionField birthdayField -
         dialogStage


  errorMessage = ""
  LF = "0a"x  
  if (nameField~getText = .nil | nameField~getText~length = 0) then
     errorMessage ||= LF"No valid name"

  if (teamField~getText = .nil | teamField~getText~length = 0) then
     errorMessage ||= LF"No valid team"

  if (positionField~getText = .nil | positionField~getText~length = 0) then
     errorMessage ||= LF"No valid position"

  if (birthdayField~getText = .nil | birthdayField~getText~length = 0) then
     errorMessage ||= LF"No valid birthday"

  if (errorMessage~length = 0) then
     return .true

  
  if .my.app~bDialogsAlertsAvailable then
  do
     alert=.fx.alert~new(.fx.Alert.Type~ERROR)
     alert~setTitle("Jadmin | Invalid Fields")
     alert~setHeaderText("Please correct the listed errors!")
     alert~setContentText("Invalid values:" LF errorMessage)
     alert~showAndWait
  end
  else  -- use BSF.CLS' .bsf.dialog utility class
  do
    .bsf.dialog~messageBox("Please correct the listed invalid fields:" LF errorMessage, "Jadmin | Invalid Fields", "error")
  end

   return .false


::class Person


   ----------------------------------------------------------------------------------------
::attribute id get
  expose id
  return id~get

::attribute id set
  expose id
  use arg val
  return id~set(val)

::attribute idProperty get
  expose id
  return id

   ----------------------------------------------------------------------------------------
::attribute name get
  expose name
  return name~get

::attribute name set
  expose name
  use arg val
  return name~set(val)

::attribute nameProperty get
  expose name
  return name

------------------------------------------------------------------------------------
::attribute team get
  expose team
  return team~get

::attribute team set
  expose team
  use arg val
  return team~set(val)

::attribute teamProperty get
  expose team
  return team

   ----------------------------------------------------------------------------------------
::attribute position get
  expose position
  return position~get

::attribute position set
  expose position
  use arg val
  return position~set(val)

::attribute positionProperty get
  expose position
  return position

   ----------------------------------------------------------------------------------------
::attribute birthday get
   expose birthday
   return birthday~get

::attribute birthday set
   expose birthday
  use arg val
   return birthday~set(val)

::attribute birthdayProperty get
   expose birthday
   return birthday

   ----------------------------------------------------------------------------------------
::method init        -- constructor
  expose id name team position birthday

  use arg strId="<enter ID>", strName="Name", -
          strTeam="Keine", -
          strPosition="Keine", strBirthday=("Format: yyyy-mm-dd")

  id=  .fx.SimpleStringProperty~new(strId)
  name =  .fx.SimpleStringProperty~new(strName)
  team= .fx.SimpleStringProperty~new(strTeam)
  position      = .fx.SimpleStringProperty~new(strPosition)
  birthday  = .fx.SimpleStringProperty~new(strBirthday)


::class RootLayoutController

::method init      
  expose mainApp -
        menuAbout menuExit
  use arg mainApp 

  rp=BSFCreateRexxProxy(self, ,"javafx.event.EventHandler")

  rlDir=.my.app~rootLayout.fxml
  menuAbout =rlDir~menuAbout ~~setOnAction(rp)
  menuExit  =rlDir~menuExit  ~~setOnAction(rp)

::method handle
  expose enuSaveAs menuAbout menuExit mainApp
  use arg event, slotDir


  tgtObjectName=event~getTarget~objectname

  select
     when tgtObjectName=menuAbout ~objectName then self~handleAbout
     when tgtObjectName=menuExit  ~objectName then self~handleExit
     otherwise .error~say("UNKNOWN target:" pp(event~getTarget) pp(event~getTarget~tostring))
  end

::method handleAbout
  expose mainApp

  lf="0a"x
  text="Jadmin - JavaFX, ooRexx & SQLite Member Administration"lf"by Emil Nusser und Thomas Schroll" lf lf"Massive thank you to Marco Jakob for the Java base and Rony G. Flatscher for the ooRexx base this app was built on"lf lf"Stylesheet by Rony G. Flatscher and Marco Jakob"

  if .my.app~bDialogsAlertsAvailable then
  do
     alert=.fx.alert~new(.fx.Alert.Type~information)  
     alert~setTitle("Jadmin | Member Administration")
     alert~setHeaderText(.nil)
     alert~setContentText(text)
     alert~showAndWait
  end
  else  -- use BSF.CLS' .bsf.dialog utility class
  do
     -- .bsf.dialog is defined in BSF.CLS and uses swing
    .bsf.dialog~messageBox(text, "Jadmin | Member Administration", "information")
  end


::method handleExit

  bsf.loadClass("javafx.application.Platform")~exit  
